// ATMEL 24C256 Serial EEPROM
// Chin-Shiuh Shieh on 2014-11-29

#define SEEPROM_DEVICE_ADDRESS 0x00

#define I2C_SCL 21
#define I2C_SDA 20

#define I2C_BUS_OK 0
#define I2C_SCL_NA 1
#define I2C_SDA_NA 2
#define I2C_ACK    3
#define I2C_NACK   4

unsigned char I2C_start(void){
  pinMode(I2C_SCL,OUTPUT);digitalWrite(I2C_SCL,HIGH);pinMode(I2C_SCL,INPUT);if(digitalRead(I2C_SCL)==LOW)return(I2C_SCL_NA);
  pinMode(I2C_SDA,OUTPUT);digitalWrite(I2C_SDA,HIGH);pinMode(I2C_SDA,INPUT);if(digitalRead(I2C_SDA)==LOW)return(I2C_SDA_NA);
  pinMode(I2C_SDA,OUTPUT);digitalWrite(I2C_SDA,LOW);pinMode(I2C_SCL,OUTPUT);digitalWrite(I2C_SCL,LOW);return(I2C_BUS_OK);
}

void I2C_stop(void){
  pinMode(I2C_SCL,OUTPUT);pinMode(I2C_SDA,OUTPUT);
  digitalWrite(I2C_SDA,LOW);digitalWrite(I2C_SCL,HIGH);digitalWrite(I2C_SDA,HIGH);
}

unsigned char I2C_check_ack(void){
  unsigned char ack;
  pinMode(I2C_SDA,INPUT);digitalWrite(I2C_SDA,HIGH);
  pinMode(I2C_SCL,OUTPUT);digitalWrite(I2C_SCL,HIGH);ack=digitalRead(I2C_SDA);digitalWrite(SCL,LOW);
  if(ack==0)return(I2C_ACK);else return(I2C_NACK);
}

unsigned char I2C_shift_out(unsigned char shift_buffer){
  unsigned char i;
  pinMode(I2C_SCL,OUTPUT);pinMode(I2C_SDA,OUTPUT);
  for(i=0;i<8;i++){
    if(shift_buffer>=128)digitalWrite(I2C_SDA,HIGH);else digitalWrite(I2C_SDA,LOW);shift_buffer<<=1;
    digitalWrite(I2C_SCL,HIGH);digitalWrite(I2C_SCL,LOW);
  }
  return(I2C_check_ack());
}

unsigned char I2C_shift_in(unsigned char *shift_buffer){
  unsigned char i;
  *shift_buffer=0;
  pinMode(I2C_SCL,OUTPUT);pinMode(I2C_SDA,INPUT);digitalWrite(I2C_SDA,HIGH);
  for(i=0;i<8;i++){
    *shift_buffer<<=1;
    digitalWrite(I2C_SCL,HIGH);if(digitalRead(I2C_SDA)==HIGH)*shift_buffer+=1;digitalWrite(I2C_SCL,LOW);
  }
  return(I2C_check_ack());
}

void seeprom_write_byte(unsigned char seeprom_device_address, unsigned int seeprom_memory_address, unsigned char seeprom_byte_data)
{
seeprom_write_byte_start:
if(I2C_start()!=I2C_BUS_OK)
  goto seeprom_write_byte_start;
if(I2C_shift_out(0xA0|(seeprom_device_address*2+0))==I2C_NACK)
  {I2C_stop();goto seeprom_write_byte_start;}
if(I2C_shift_out(seeprom_memory_address/256)==I2C_NACK)
  {I2C_stop();goto seeprom_write_byte_start;}
if(I2C_shift_out(seeprom_memory_address%256)==I2C_NACK)
  {I2C_stop();goto seeprom_write_byte_start;}
if(I2C_shift_out(seeprom_byte_data)==I2C_NACK)
  {I2C_stop();goto seeprom_write_byte_start;}
I2C_stop();
}

unsigned char seeprom_read_byte(unsigned char seeprom_device_address, unsigned int seeprom_memory_address)
{
unsigned char seeprom_byte_data;
seeprom_read_byte_start:
if(I2C_start()!=I2C_BUS_OK)
  goto seeprom_read_byte_start;
if(I2C_shift_out(0xA0|(seeprom_device_address*2+0))==I2C_NACK)
  {I2C_stop();goto seeprom_read_byte_start;}
if(I2C_shift_out(seeprom_memory_address/256)==I2C_NACK)
  {I2C_stop();goto seeprom_read_byte_start;}
if(I2C_shift_out(seeprom_memory_address%256)==I2C_NACK)
  {I2C_stop();goto seeprom_read_byte_start;}
if(I2C_start()!=I2C_BUS_OK)
  goto seeprom_read_byte_start;
if(I2C_shift_out(0xA0|(seeprom_device_address*2+1))==I2C_NACK)
  {I2C_stop();goto seeprom_read_byte_start;}
I2C_shift_in(&seeprom_byte_data);
I2C_stop();
return seeprom_byte_data;
}	

void setup(void){
  Serial.begin(9600);
}

void loop(void){
unsigned int seeprom_memory_address;
for(seeprom_memory_address=0;seeprom_memory_address<16;seeprom_memory_address+=2){
  seeprom_write_byte(SEEPROM_DEVICE_ADDRESS,seeprom_memory_address,(seeprom_memory_address/256));
  seeprom_write_byte(SEEPROM_DEVICE_ADDRESS,seeprom_memory_address+1,(seeprom_memory_address%256));
}
for(seeprom_memory_address=0;seeprom_memory_address<16;seeprom_memory_address+=2){
  Serial.print(seeprom_read_byte(SEEPROM_DEVICE_ADDRESS,seeprom_memory_address),HEX);
  Serial.print(seeprom_read_byte(SEEPROM_DEVICE_ADDRESS,seeprom_memory_address+1),HEX);
}
while(1==1);
}
